def ubx_checksum(payload):
  ck_a = ck_b = i = 0
  while i < len(payload):
    ck_a = (ck_a + payload[i]) & 0xff
    ck_b = (ck_b + ck_a) & 0xff
    i += 1
  return ck_a, ck_b

def ubx2dcr(message):
  ck_a, ck_b = ubx_checksum(message[2:-2])
  if message[-2] == ck_a and message[-1] == ck_b:
    if message[:7] == b'\xB5\x62\x02\x13\x2C\x00\x05':
      dwords = message[14:]
      swapped = b''
      for i in range(9):
        swapped += bytes((dwords[3+i*4], dwords[2+i*4],
                          dwords[1+i*4], dwords[0+i*4]))
      if swapped[1] >> 2 == 43 or swapped[1] >> 2 == 44:
        dcr_message = (
          swapped[:31] + bytes((swapped[31] & 0xC0,))
        ).hex()[:-1]
        return dcr_message.upper()

import argparse
import time
import serial

if __name__ == '__main__':
  parser = argparse.ArgumentParser(description='DC Report')
  parser.add_argument('port', help='Serial port')
  args = parser.parse_args()

  with serial.Serial(args.port, 115200) as ser:
    print('Start!')
    while True:
      line = b''
      ubx_flag = False
      count = length = 0
      while True:
        if ubx_flag:
          if count > 4 and length == 0:
            length = int.from_bytes(line[4:5], "little")
          if length > 0 and count == length + 8:
            break
        b = ser.read()
        if b == b'\x62' and line == b'\xB5':
          ubx_flag = True
        if b == b'\n':
          if line.endswith(b'\r'):
            line += b
            break
        else:
          line += b
        count += 1

      if ubx_flag:
        message = ubx2dcr(line)
        if message:
          print(message)
